@@ -56,7 +56,6 @@ struct _DhWindowPriv {
         GtkWidget      *findbar;
 
         GtkBuilder     *builder;
-        GtkActionGroup *action_group;
 
         DhLink         *selected_search_link;
         guint           find_source_id;
@@ -385,6 +384,16 @@ zoom_default_cb (GSimpleAction *action,
         window_update_zoom_actions_state (window);
 }
 
+static void
+focus_search_cb (GSimpleAction *action,
+                 GVariant      *parameter,
+                 gpointer       user_data)
+{
+        DhWindow *window = user_data;
+
+        dh_sidebar_set_search_focus (DH_SIDEBAR (window->priv->sidebar));
+}
+
 static void
 go_back_cb (GSimpleAction *action,
             GVariant      *parameter,
@@ -424,6 +433,19 @@ go_forward_cb (GSimpleAction *action,
         webkit_web_view_go_forward (web_view);
 }
 
+static void
+gear_menu_cb (GSimpleAction *action,
+              GVariant      *parameter,
+              gpointer       user_data)
+{
+        GVariant *state;
+
+        state = g_action_get_state (G_ACTION (action));
+        g_action_change_state (G_ACTION (action),
+                               g_variant_new_boolean (!g_variant_get_boolean (state)));
+        g_variant_unref (state);
+}
+
 static void
 window_open_link_cb (DhWindow *window,
                      const char *location,
@@ -451,9 +473,12 @@ static GActionEntry win_entries[] = {
         { "zoom-in",          zoom_in_cb,          NULL, NULL, NULL },
         { "zoom-out",         zoom_out_cb,         NULL, NULL, NULL },
         { "zoom-default",     zoom_default_cb,     NULL, NULL, NULL },
+        { "focus-search",     focus_search_cb,     NULL, NULL, NULL },
         /* go */
         { "go-back",          go_back_cb,          NULL, "false", NULL },
         { "go-forward",       go_forward_cb,       NULL, "false", NULL },
+        /* gear menu */
+        { "gear-menu",        gear_menu_cb,        NULL, "false", NULL },
 };
 
 static void
@@ -668,6 +693,8 @@ window_populate (DhWindow *window)
                                                 "emblem-system-symbolic",
                                                 "",
                                                 FALSE);
+        gtk_actionable_set_action_name (GTK_ACTIONABLE (menu_button), "win.gear-menu");
+
         menu = gtk_builder_get_object (priv->builder, "window-menu");
         gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (menu_button), G_MENU_MODEL (menu));
 
@@ -734,6 +761,9 @@ window_populate (DhWindow *window)
 
         gtk_widget_show_all (priv->hpaned);
 
+        /* Focus search in sidebar by default */
+        dh_sidebar_set_search_focus (DH_SIDEBAR (priv->sidebar));
+
         window_update_zoom_actions_state (window);
         window_open_new_tab (window, NULL, TRUE);
 }
@@ -903,7 +933,7 @@ static gboolean
 window_web_view_load_error_cb (WebKitWebView  *web_view,
                                WebKitWebFrame *frame,
                                gchar          *uri,
-                               gpointer       *web_error,
+                               GError         *web_error,
                                DhWindow       *window)
 #endif
 {
@@ -913,6 +943,10 @@ window_web_view_load_error_cb (WebKitWebView  *web_view,
         GList     *children;
         gchar     *markup;
 
+        /* Ignore cancellation errors; which happen when typing fast in the search entry */
+        if (g_error_matches (web_error, WEBKIT_NETWORK_ERROR, WEBKIT_NETWORK_ERROR_CANCELLED))
+                return TRUE;
+
         info_bar = window_get_active_info_bar (window);
         markup = g_strdup_printf ("<b>%s</b>",
                        _("Error opening the requested link."));
